function Buckley_Leverett_MatMOL(eps0,n0,orderDconv,orderDdiff,Reltol,Abstol)
%
% The MatMol Group (2009)
% Function....... : Transient simulation of the classic 1D Buckley-Leverett
%                   equation for oil reservoirs
% Technique...... : Solution with the MatMOL toolbox
% Input...........: eps0:     cappilary pressure term
%                   n0:       number of vertex centered discritisation points
%                   orderDconv:  approximation order for convective term
%                   orderDdiff:  approximation order for diffusive term
%                   RelTol:   Relatitve integration tolerance
%                   AbsTol:   Absolute integration tolerance

close all

global eps
global n Dz Dzc

if nargin < 1, eps0         =   0.01;   end  
if nargin < 2, n0           =   101;    end  
if nargin < 3, orderDconv   =   3;      end  
if nargin < 4, orderDdiff   =   2;      end  
if nargin < 5, Reltol       =   1.0e-4; end  
if nargin < 6, Abstol       =   1.0e-4; end  

tic % initialise clock


% 1 Parameters
% ************

% Process
eps     =   eps0;               % [-]       capillary pressure term
clear eps0

% Spatial grid
n   	=   n0;                 % [-]       # of discretisation points
zL      =   0.0;                % [-]       left border position
zR      =   1.0;                % [-]       right border position
dz      =   (zR-zL)/(n-1);      % [-]       spatial discretisation length 
Z       =   zL:dz:zR;           % [-]       dimensionless position vector

% Temporal grid
t_start =   0.0;                % [-]       initial time
t_end   =   0.4;                % [-]       final time
dt_plot =   0.01;               % [-]       time interval for plots


% 2 Numerical computation
% ***********************

% 2a Differentiation matrices
switch orderDconv
    case 1
        Dzc = two_point_upwind_uni_D1(zL,zR,n,1);
    case 3
        Dzc = four_point_biased_upwind_uni_D1(zL,zR,n,1);
    otherwise
        disp('Invalid selection for order of approximation of convective term.')
        disp('Choose 1 for low-order or 3 for high-order scheme.')
end

switch orderDdiff
    case 2
        Dz  = three_point_centered_uni_D1(zL,zR,n);
    case 4
        Dz  = five_point_centered_uni_D1(zL,zR,n);
    otherwise
        disp('Invalid selection for order of approximation of diffusive term.')
        disp('Choose 2 for low-order or 4 for high-order scheme.')
end

% 2b Initial conditions
S0 = max(zeros(1,n),interp1([0 1/3 1],[1 0 0],Z));
IC = [S0];
clear S0
    
% 2c Solution of the DAEs
options = odeset('Reltol',Reltol,'Abstol',Abstol,'JPattern',Jpattern_BL_MatMOL,...
          'Jacobian',@Jacobian_BL_MatMOL);      
sol = ode15s(@ODE_BL_MatMOL,[t_start:dt_plot:t_end],IC',options);

% 2d Reactor properties
t    = t_start:dt_plot:t_end;   % time points to be plotted
sol2 = deval(sol,t);            % evaluate solution at time points
S    = sol2(1:n,:);



% 2e Integrator statisctics
Stats   = sol.stats; % solution statistics
CPUtime = toc;       % read clock
clear sol sol2

%disp('Integration statistics:')
fprintf('  %g successful steps\n', Stats.nsteps);
fprintf('  %g failed attempts\n',  Stats.nfailed);
fprintf('  %g function evaluations\n', Stats.nfevals);
fprintf('  %g partial derivatives\n',  Stats.npds);
fprintf('  %g LU decompositions\n', Stats.ndecomps);
fprintf('  %g solutions of linear systems\n', Stats.nsolves);
fprintf('  %g s (total) computation time\n',  CPUtime);


% 3 Plot results
% **************

figure(1)
mesh(t,Z,S);
ylabel('Space [-]','Fontsize',18)
xlabel('Time [-]','Fontsize',18)
zlabel('Saturation [-]','Fontsize',18)
title(['MatMOL: \epsilon=' num2str(eps) ', n=' num2str(n)],'Fontsize',18)
axis([t_start t_end zL zR -0.1 1.1])
grid on
set(gca,'Fontsize',18,'Ydir','rev')

figure(2)
plot(Z,S);
xlabel('Space [-]','fontsize',18);
ylabel('Saturation [-]','Fontsize',18)
title(['MatMOL: \epsilon=' num2str(eps) ', n=' num2str(n) ', RT=' ...
        num2str(Reltol) ', AT=' num2str(Abstol)],'Fontsize',18)
axis([zL zR -0.1 1.1])
grid on
set(gca,'Fontsize',18)
   
end % End of function Buckley_Leverett_MatMOL
%**************************************************************************

function dX = ODE_BL_MatMOL(t,X)
% Provides the right hand side of the ODEs

global eps
global n Dz Dzc

% Independent variables
S   = X;

% Spatial derivatives
Sz  = Dz*S;
f1  = 4*S.*(1-S).*Sz;
f1z = Dz*f1;

f2  = (S.^2)./(S.^2+((1-S).^2));
Szc = Dzc*f2;

% Temporal derivative
St  = -Szc + eps*f1z;

% RHS of ODEs
dX  = [St];
     
end % End of function ODE_BL_MatMOL
%**************************************************************************

function Jp = Jpattern_BL_MatMOL
% Provides the sparsity pattern of the Jacobian.

global n Dz Dzc

M   = diag(abs(Dz)*ones(n,1)) + diag(ones(1,n))*abs(Dz);
Jp1 = abs(Dz)*M;
Jp2 = abs(Dzc)*diag(ones(n,1));

Jp  = sparse(sign(Jp1+Jp2)); 

end % End of function Jpattern_BL_MatMOL
%**************************************************************************

function dFdX = Jacobian_BL_MatMOL(t,X)
% Provides the Jacobian of the right hand side of the ODEs
     
global eps
global n Dz Dzc

% Independent variables
S = X;

% Jacobian of RHS of ODE
f1  = 4*S.*(1-S);
f1z = -8*S+4;
M   = diag(f1z.*(Dz*S)) + diag(f1)*Dz;
dF1dX = Dz*M;

f2  = (S.^2)./(S.^2+((1-S).^2));
f2z = diag(2*S.*(1-S)./((S.^2+((1-S).^2)).^2)); 
dF2dX = Dzc*f2z;

dFdX = sparse(-dF2dX + eps*dF1dX); 

end % End of function Jacobian_BL_MatMOL
