function Buckley_Leverett_MatMOL_FluxLimiter(eps0,n0,orderDdiff,Reltol,...
                                             Abstol,FluxLimiter0)
%
% The MatMol Group (2009)									
% Function....... : Transient simulation of the classic 1D Buckley-Leverett
%                   equation for oil reservoirs
% Technique...... : Solution with the MatMOL toolbox and FluxLimiter
% Input...........: eps0:     cappilary pressure term
%                   n0:       number of vertex centered discritisation points
%                   orderDdiff:  approximation order for diffusive term
%                   RelTol:   Relatitve integration tolerance
%                   AbsTol:   Absolute integration tolerance
%                   FluxLimiter0: choice of flux limiting function:
%                                 Koren, Mc, Minmod, Smart, Superbee or 
%                                 VanLeer

close all

global eps
global n Dz Z FluxLimiter

if nargin < 1, eps0         =   0.001;      end  
if nargin < 2, n0           =   101;        end  
if nargin < 3, orderDdiff   =   2;          end  
if nargin < 4, Reltol       =   1.0e-4;     end  
if nargin < 5, Abstol       =   1.0e-4;     end 
if nargin < 6, FluxLimiter0 =   'VanLeer';  end 

tic % initialise clock


% 1 Parameters
% ************

% Process
eps     =   eps0;               % [-]       capillary pressure term
clear eps0

% Spatial grid
n   	=   n0;                 % [-]       # of discretisation points
zL      =   0.0;                % [-]       left border position
zR      =   1.0;                % [-]       right border position
dz      =   (zR-zL)/(n-1);      % [-]       spatial discretisation length 
Z       =   zL:dz:zR;           % [-]       dimensionless position vector

% Temporal grid
t_start =   0.0;                % [-]       initial time
t_end   =   0.4;                % [-]       final time
dt_plot =   0.01;               % [-]       time interval for plots

FluxLimiter = FluxLimiter0
clear FluxLimiter0


% 2 Numerical computation
% ***********************

% 2a Differentiation matrices
switch orderDdiff
    case 2
        Dz  = three_point_centered_uni_D1(zL,zR,n);
    case 4
        Dz  = five_point_centered_uni_D1(zL,zR,n);
    otherwise
        disp('Invalid selection for order of approximation of diffusive term.')
        disp('Choose 2 or 4 for high-order scheme.')
end

% 2b Initial conditions
S0 = max(zeros(1,n),interp1([0 1/3 1],[1 0 0],Z));
IC = [S0];
clear S0
    
% 2c Solution of the DAEs
options = odeset('Reltol',Reltol,'Abstol',Abstol,...
    'JPattern',Jpattern_BL_FluxLimiter); 

sol = ode15s(@ODE_BL_FluxLimiter,[t_start:dt_plot:t_end],IC',options);

% 2d Reactor properties
t    = t_start:dt_plot:t_end;   % time points to be plotted
sol2 = deval(sol,t);            % evaluate solution at time points
S    = sol2(1:n,:);

% 2e Integrator statisctics
Stats   = sol.stats; % solution statistics
CPUtime = toc;       % read clock
clear sol sol2

%disp('Integration statistics:')
fprintf('  %g successful steps\n', Stats.nsteps);
fprintf('  %g failed attempts\n',  Stats.nfailed);
fprintf('  %g function evaluations\n', Stats.nfevals);
fprintf('  %g partial derivatives\n',  Stats.npds);
fprintf('  %g LU decompositions\n', Stats.ndecomps);
fprintf('  %g solutions of linear systems\n', Stats.nsolves);
fprintf('  %g s (total) computation time\n',  CPUtime);


% 3 Plot results
% **************

figure(1)
mesh(t,Z,S);
ylabel('Space [-]','Fontsize',18)
xlabel('Time [-]','Fontsize',18)
zlabel('Saturation [-]','Fontsize',18)
title(['MatMOL+' FluxLimiter ': \epsilon=' num2str(eps) ', n=' ...
       num2str(n)],'Fontsize',20)
axis([t_start t_end zL zR -0.1 1.1])
grid on
set(gca,'Fontsize',18,'Ydir','rev')

figure(2)
plot(Z,S);
xlabel('Space [-]','fontsize',18);
ylabel('Saturation [-]','Fontsize',18)
title(['MatMOL+' FluxLimiter ': \epsilon=' num2str(eps) ', n=' ...
       num2str(n) ', RT=' num2str(Reltol) ', AT=' num2str(Abstol)],'Fontsize',18)
axis([zL zR -0.1 1.1])
grid on
set(gca,'Fontsize',18)
   
end % End of function Buckley_Leverett_FluxLimiter
%**************************************************************************
function dX = ODE_BL_FluxLimiter(t,X)
% Provides the right hand side of the ODEs

global eps
global n Dz Z FluxLimiter

% Independent variables
S=X;

% Fluxlimiters
switch FluxLimiter
    case 'Koren'
        fx = koren_slope_limiter_fz(Z,n,t,S,@flux,@dflux_dx);
    case 'MC'
        fx = mc_slope_limiter_fz(Z,n,t,S,@flux,@dflux_dx);
    case 'Minmod'
        fx = minmod_slope_limiter_fz(Z,n,t,S,@flux,@dflux_dx);
    case 'Smart'
        fx = smart_slope_limiter_fz(Z,n,t,S,@flux,@dflux_dx);
    case 'Superbee'
        fx = superbee_slope_limiter_fz(Z,n,t,S,@flux,@dflux_dx);
    case 'VanLeer'
        fx = vanleer_slope_limiter_fz(Z,n,t,S,@flux,@dflux_dx);
    otherwise
        disp('Wrong choice of flux limiter. Valid choices are: ')
        disp('Koren, MC, Minmod, Smart, Superbee and VanLeer')      
        disp('Provided as strings.')
end

% Spatial derivatives
Sz  = Dz*S;
f1  = 4*S.*(1-S).*Sz;
f1z = Dz*f1;

% Temporal derivative
St = -fx + eps*f1z;

% RHS of ODEs
dX = [St];
     
end % End of function ODE_BL_FluxLimiter
%**************************************************************************

function fx = flux(t,X)
% Computes the flux

fx = (X.^2)./(X.^2+((1-X).^2));

end % End of function flux
%**************************************************************************

function dfxdx = dflux_dx(t,X)
% Computes the flux gradient 

dfxdx = 2*X.*(1-X)./((X.^2+((1-X).^2)).^2);

end % End of function dflux
%**************************************************************************

function Jp = Jpattern_BL_FluxLimiter
% Provides the sparsity pattern of the Jacobian assuming high order
% discretisation schemes

global n

e  = ones(n,1);
Jp = spdiags([e e e e e e e e e],[-4 -3 -2 -1 0 1 2 3 4],n,n);

end % End of function Jpattern_BL_FluxLimiter
%**************************************************************************
