function TDFR_MatMOL_DynamicRegridding(Pe,n0,zswitch,Reltol,Abstol)
%
% The MatMol Group (2009)
% Function....... : Transient simulation of a jacketed tubular reactor with
%                   with dispersion and Danckwerts boundary conditions.
%                   "F. Logist, I. Smets, A. Vande Wouwer, J. Van Impe 2005.
%                   Optimal control of dispersive tubular chemical reactors:
%                   Part II" Procs of 16th IFAC World Congress, DVD-ROM, 6p.
% Technique...... : Solution with the MatMOL toolbox and dynamic regridding
%                   MOVGRID
% Input...........: Pe:       Peclet number
%                   n0:       number of vertex centered discritisation points 
%                   zswitch:  position where the jacket temperature switches 
%                             from its maximum to its minimum value 
%                   RelTol:   Relatitve integration tolerance
%                   AbsTol:   Absolute integration tolerance

% Global variables
global zL zR n ne X eq choice
global alpha kappa mu tau
global u z
global z1 Twmax Twmin D1 Tin Cin delta1 k0 E R beta1 D2 v

if nargin < 1, Pe       = 1.0e+6;   end
if nargin < 2, n0       = 51;       end
if nargin < 3, zswitch  = 0.54;     end
if nargin < 4, Reltol   = 1.0e-4;   end
if nargin < 5, Abstol   = 1.0e-6;   end

tic % initialise clock


% Model Parameters
v     =  0.1;        % [m/s]           fluid velocity
L     =  1.0;        % [m]             reactor length
Pe1   =  Pe;         % [-]             Peclet number
Pe2   =  Pe;         % [-]             Peclet number
D1    =  v*L/Pe1;    % [m^2/s]         dispersion coefficient
D2    =  v*L/Pe2;    % [m^2/s]         dispersion coefficient
Tin   =  340.0;      % [K]             feed temperature
Twmin =  280.0;      % [K]             minimum jacket temperature
Twmax =  400.0;      % [K]             maximum jacket temperature
Cin   =  0.02;       % [mol/L]         feed concentration
z1    =  zswitch;    % [m]             switching position
E     =  11250.0;    % [cal/mol]       activation energy
R     =  1.986;      % [cal/(mol*K)]   gass constant
k0    =  1.0e+6;     % [1/s]           rate constant       

% Note: 
% To avoid confusion with parameters from the regridding code, the reactor 
% related parameters beta and delta are given an additional index 1:
beta1  = 0.2;                % [1/s]     dimensionless heat transfer
delta1 = 0.25;               % [-]       dimensionless heat of reaction

% Spatial grid
n   =   n0-2;                      % [-]   # of moving grid points
zL  =   0;                         % [m]   left border
zR  =   L;                         % [m]   right border
dz  =   (zR-zL)/(n+1);             % [m]   spatial discretisation length
z   =   [zL:dz:zswitch-dz/2 ...    % [m]   position vector
         zswitch:dz:zR-dz/2 zR]'; 
% Note:
% If n0 is the original number of discretisation points (i.e., dim(z)), 
% n is the number of moving grid points. (z0 and zL are fixed.)

% Temporal grid
t_start = 0;                      % [s]   initial time
t_end   = 20;                     % [s]   final time
dt_plot = 1;                      % [s]   time interval for plots
t_plot  = t_start:dt_plot:t_end;  % [s]   time points to be plotted
     
% Parameters of the moving grid (MOVGRID)
ne      =   2;                  % [-]       nr of PDEs
alpha   =   0.1;                % [-]
kappa   =   2;                  % [-]
mu      =   kappa*(kappa+1);    % [-]
tau     =   1.0e-7;             % [-]
choice  =   'der1';             % [-]       monitoring function choice

% Initial conditions for the dependent variables
X10 = Tin*ones(length(z),1);
X20	= 0.0*ones(length(z),1);
u   = [X10 , X20];
for jj = 1:ne   % Reorder u to x(n+2,ne)
    x(jj:ne+1:n*(ne+1)) = u(2:n+1 , jj);
end
x(ne+1:ne+1:n*(ne+1))=z(2:n+1);
clear X10 X20 

% Global variables for JPat : 
%  Function JPat implements the sparsity pattern of the jacobian in X ;
%  informations depending on the choosen spatial derivative matrices are
%  stocked in the ne eq(ne,ne,ne) matrices. 
X = zeros(n*(ne+1),n*(ne+1));

% 2D and 1D spatial derivatives by 5 pts centered finite difference
eq(:,:,1) = [-2 2 1; 0 0 1];
eq(:,:,2) = [ 0 0 1;-2 2 1];

% Monitoring choice :
% Choose between :
%                                     1    ne (u(i+1,j)-u(i,j))^2
%     der1'   : m(i) = sqrt[ alpha + ---  sum ------------------- ]
%                                     ne  j=1   (z(i+1)-z(i))^2
%        
%                                     1    ne     uzz(i+1,j)+uzz(i,j)
%     'der2'  : m(i) = sqrt[ alpha + ---  sum abs(-------------------)]
%                                     ne  j=1           2  
%
%                                     1    ne (fl[u(i+1,j)]-fl[u(i,j)])^2
%     'derfl' : m(i) = sqrt[ alpha + ---  sum --------------------------- ]
%                                     ne  j=1      (z(i+1)-z(i))^2
choice = 'der1';

% Global initial condition
for jj = 1 : ne,
    x(jj : ne+1 : n*(ne+1)) = u(2 : n+1 , jj);
end
x(ne+1 : ne+1 : n*(ne+1))   = z(2 : n+1);

%  Ode solver parameters
options = odeset('RelTol',Reltol,'AbsTol',Abstol,'Mass',@mass,...
                 'MStateDependence','strong','JPattern',JPat,...
                 'MvPattern',MvPat,'MassSingular','no','BDF','on',...
                 'MaxOrder',5);

% Call the ODE solver
sol = ode15s(@right_vect,t_plot,x,options);

% Integrator statisctics
Stats   = sol.stats; % solution statistics
CPUtime = toc;       % read clock
disp('Integration statistics:')
fprintf('  %g successful steps\n', Stats.nsteps);
fprintf('  %g failed attempts\n',  Stats.nfailed);
fprintf('  %g function evaluations\n', Stats.nfevals);
fprintf('  %g partial derivatives\n',  Stats.npds);
fprintf('  %g LU decompositions\n', Stats.ndecomps);
fprintf('  %g solutions of linear systems\n', Stats.nsolves);
fprintf('  %g s (total) computation time\n',  CPUtime);

% Plot the solution
sol2  = deval(sol,t_plot);         % evaluate solution at time points
yout  = sol2';
t     = t_plot;
nt    = length(t);
absc  = zeros(n+2,nt);
u     = zeros(n+2,nt);
clear sol2

% Separate the dependent variables and the node positions
for ii = 1:nt
    % Spatial grid at all times
    Z(2:n+1,ii) = yout(ii,ne+1:ne+1:n*(ne+1));
    Z(1,ii)     = zL;
    Z(n+2,ii)   = zR;
    Dz            = matfd(Z(:,ii), 'non_uni', '1st', 5, 'centered');
    % Field T
    T(2:n+1,ii)    = yout(ii,1:ne+1:(n)*(ne+1)-ne);
    T(1,ii)        = BC_dir('rob',T(:,ii),Tin,Dz,'begin',v,D1);  % First point
    T(n+2,ii)      = BC_dir('neu',T(:,ii),0,Dz,'end',0,0);      % Last point
    % Field C
    C(2:n+1,ii)    = yout(ii,2:ne+1:(n)*(ne+1)-ne+1);
    C(1,ii)        = BC_dir('rob',C(:,ii),Cin,Dz,'begin',v,D2);  % First point
    C(n+2,ii)      = BC_dir('neu',C(:,ii),0,Dz,'end',0,0);      % Last point
end

% Field representation
figure(1)
subplot('position',[0.1 0.3 0.8 0.6])
plot(Z,T,'g',Z,T,'.k')
ylabel('Temperature T [K]','Fontsize',18)
title(['MatMOL Dynamic Regridding: Pe1=Pe2=' num2str(Pe2)],'Fontsize',18)
axis([zL zR 270.0 450.0])
set(gca,'Fontsize',16)

subplot('position',[0.1 0.1 0.8 0.15])
for i=1:length(t),
    plot(Z(:,i),t_plot(i)*ones(1,n+2),'.k')
    hold on
end
xlabel('Space z [m]','fontsize',18);
ylabel('Time [s]','Fontsize',18)
axis([zL zR t_start t_end])
set(gca,'Fontsize',16)
    
figure(2)
subplot('position',[0.1 0.3 0.8 0.6])
plot(Z,C,'g',Z,C,'.g')
ylabel('Concentration C [mole/K]','Fontsize',18)
title(['MatMOL Dynamic Regridding: Pe1=Pe2=' num2str(Pe2)],'Fontsize',18)
axis([zL zR -0.001 0.02])
set(gca,'Fontsize',16)
    
subplot('position',[0.1 0.1 0.8 0.15])
for i=1:length(t),
    plot(Z(:,i),t_plot(i)*ones(1,n+2),'.k')
    hold on
end
xlabel('Space z [m]','fontsize',18);
ylabel('Time [s]','Fontsize',18)
axis([zL zR t_start t_end])
set(gca,'Fontsize',16)